import java.awt.*;
import java.awt.event.*;

public class Main extends Frame implements ActionListener {
    Label statusBar = new Label("", Label.CENTER);
    TypingTutor TypingTutor = new TypingTutor(statusBar);
    Button startButton = new Button("Start");

    Main() {
        super("KeyAdapter Example");

        // Listen for events
        startButton.addActionListener(this);

        // Layout components
        add(startButton, BorderLayout.NORTH);
        add(statusBar, BorderLayout.SOUTH);
        add(TypingTutor, BorderLayout.CENTER);
        pack();
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        TypingTutor.requestFocus();
        TypingTutor.start();
    }

    public static void main(String args[]) {
        new Main();
    }
}

class TypingTutor extends Canvas implements Runnable {
    int width = 200;
    int height = 300;
    Label statusBar;
    int score;

    // Reference to the timer thread.
    Thread timer;

    // Unless true, the start() method is ignored.
    boolean done = true;

    // Locations and value of letters.
    Point[] letterPts;
    char[] letters;

    // Double buffer variables
    Image bbuf;
    Graphics bbufG;
    
    // Font variables
    Font f = new Font("Monospaced", Font.BOLD, 20);
    FontMetrics fm;

    TypingTutor(Label statusBar) {
        this.statusBar = statusBar;
        addKeyListener(new KeyEventHandler());
        addFocusListener(new FocusEventHandler());
        addMouseListener(new MouseEventHandler());
    }

    // This method should be called only when the component
    // is visible.
    synchronized void start() {
        // Initialize font metrics if necessary.
        if (fm == null) {
            fm = getFontMetrics(f);
        }
        // Create the double buffer if necessary.
        if (bbuf == null) {
            bbuf = createImage(width, height);
            bbufG = bbuf.getGraphics();
        }

        // Set variables
        newLetterProbability = .05;
        letterPts = new Point[200];
        letters = new char[200];
        score = 0;
        statusBar.setText(""+score);

        done = false;
        (timer = new Thread(this)).start();
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        if (letterPts == null) {
            return;
        }
        // Clear the background.
        bbufG.setColor(Color.yellow);
        bbufG.fillRect(0, 0, width, height);
        bbufG.setColor(Color.black);
        bbufG.drawRect(0, 0, width-1, height-1);
        bbufG.setFont(f);

        // Paint all the letters.
        for (int i=0; i<letters.length; i++) {
            if (letterPts[i] != null) {
                bbufG.drawChars(letters, i, 1, 
                    letterPts[i].x, letterPts[i].y);
            }
        }

        // Draw the off-screen image to the display.
        g.drawImage(bbuf, 0, 0, this);
    }

    public Dimension getPreferredSize() {
        return new Dimension(width, height);
    }

    // Key events.
    class KeyEventHandler extends KeyAdapter {
        public void keyTyped(KeyEvent evt) {
            int hiY = 0;
            int ltr = -1;

            // Find the lowest copy of that letter.
            for (int i=0; i<letters.length; i++) {
                if (letterPts[i] != null 
                  && letters[i] == Character.toUpperCase(evt.getKeyChar())) {
                    if (letterPts[i].y > hiY) {
                        hiY = letterPts[i].y;
                        ltr = i;
                    }
                }
            }
            
            // If the letter is found, erase it.
            if (ltr >= 0) {
                letterPts[ltr] = null;
                score++;
                statusBar.setText(""+score);

                // Increase the probability of creating new letters
                newLetterProbability += .005;
                repaint();
            }
        }
    }

    // Focus events.
    class FocusEventHandler extends FocusAdapter {
        public void focusLost(FocusEvent evt) {
            timer = null;
        }
    }

    // Mouse events.
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            requestFocus();
            (timer = new Thread(TypingTutor.this)).start();
        }
    }

    // The duration of each tick.
    int delay = 120;

    // The probability of a new letter being created at each tick.
    double newLetterProbability;

    public void run() {
        int count = 0;
        try {
            while (!done && timer == Thread.currentThread()) {
                for (int i=0; i<letters.length; i++) {
                    if (letterPts[i] != null) {
                        letterPts[i].y += 3;

                        // Check if the letter is at the bottom
                        if (letterPts[i].y >= height) {
                            // If so, clear all letters
                            for (int j=0; j<letters.length; j++) {
                                letterPts[j] = null;
                                done = true;
                            }
                        }
                    }
                }
                if (Math.random() < newLetterProbability) {
                    // Find a slot
                    for (int i=0; i<letters.length; i++) {
                        if (letterPts[i] == null) {
                            // Create a starting position for the letter.
                            letterPts[i] = new Point(
                                (int)Math.floor(Math.random()*width), 0);
                            letterPts[i].x = Math.min(letterPts[i].x, 
                                width-fm.charWidth('X'));

                            // Pick a letter
                            letters[i] = (char)('A' 
                                + Math.floor(Math.random()*26));
                            break;
                        }
                    }
                }
                repaint();
                Thread.sleep(delay);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
