import java.awt.*;
import java.awt.event.*;
import java.io.*;

class Main extends Frame implements TextListener, ActionListener {
    String filename;
    Button saveButton = new Button("Save");
    TextArea textArea = new TextArea();

    Main(String filename) {
        super("TextListener Example");
        this.filename = filename;

        try {
            char[] buffer = new char[1024];
            int count = 0;
            FileReader rd = new FileReader(filename);

            // Read in the contents of the file
            // and append them to the text area.
            while ((count = rd.read(buffer, 0, 1024)) != -1) {
                textArea.append(new String(buffer, 0, count));
            }
            rd.close();
        } catch (Exception e) {
            System.err.println("Could not open: " + filename);
            System.exit(1);
        }
        
        // Listen for events.
        textArea.addTextListener(this);
        saveButton.addActionListener(this);
        saveButton.setEnabled(false);

        // Layout components.
        add(textArea, BorderLayout.CENTER);
        add(saveButton, BorderLayout.SOUTH);
        setFont(new Font("Monospaced", Font.PLAIN, 12));
        pack();
        show();
    }

    // Handler for text events.
    public void textValueChanged(TextEvent evt) {
        // No need to listen for more text change events 
        // so remove from listener list.
        textArea.removeTextListener(this);

        saveButton.setEnabled(true);
    }
    
    // Handler for action events.
    public void actionPerformed(ActionEvent evt) {
        // Listen for text change events again.
        textArea.addTextListener(this);

        saveButton.setEnabled(false);

        // Now save the changes back to the file.
        try {
            FileWriter wr = new FileWriter(filename);
            String contents = textArea.getText();

            wr.write(contents, 0, contents.length());
            wr.close();
        } catch (Exception e) {
            System.err.println("Could not save: " + filename);
        }
    }
    
    public static void main(String args[]) {
        if (args.length != 1) {
            System.err.println("Usage: java Main <filename>");
            System.exit(1);
        }

        new Main(args[0]);
    }
}
