import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

class Main extends Frame {
    Main(String filename) {
        super("PixelGrabber Example");
        Label label = new Label();

        add(label, BorderLayout.NORTH);
        add(new ImageCanvas(getToolkit().getImage(filename), label), 
            BorderLayout.CENTER);
        setSize(300, 300);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image image;
    Label label;
    int[] pixels;
    boolean paintTargetPixels;
    int targetPixelValue;

    ImageCanvas(Image image, Label label) {
        this.image = image;
        this.label = label;

        // Add listeners for canvas
        addMouseMotionListener(new MouseMotionEventHandler());
        addMouseListener(new MouseEventHandler());
    }

    void paintPixels(Graphics g, int w, int h) {
        int count = 0;
        g.setColor(Color.red);
        for (int x=0; x<w; x++) {
            for (int y=0; y<h; y++) {
                int p = pixels[y * w + x];
                if (p == targetPixelValue) {
                    g.fillRect(x, y, 1, 1);
                    count++;
                }
            }
        }
        label.setText("count: " + count);
    }

    public void update(Graphics g) {
        paint(g);
    }
    public void paint(Graphics g) {
        if (image != null) {
            g.drawImage(image, 0, 0, this);

            int w = image.getWidth(this);
            int h = image.getHeight(this);
            if (w < 0 || h < 0) return;
            if (paintTargetPixels) {
                paintPixels(g, w, h);
            }

            if (pixels == null) {
                // Create a pixel grabber and retrieve the pixels.
                pixels = new int[w * h];
                try {
                    PixelGrabber pg = new PixelGrabber(
                        image, 0, 0, w, h, pixels, 0, w);
                    pg.grabPixels();

                    // Check for errors.
                    if ((pg.status() & ImageObserver.ABORT) != 0) {
                        System.err.println("Error while fetching image");
                        System.exit(1);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    System.exit(1);
                }
            }
        }
    }

    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseMoved(MouseEvent evt) {
            int x = evt.getX(), y = evt.getY();

            if (pixels == null ||
                x >= image.getWidth(ImageCanvas.this) || 
                y >= image.getHeight(ImageCanvas.this)) {
                label.setText("");
                return; 
            }
            ColorModel cm = ColorModel.getRGBdefault();
            int p = pixels[y * image.getWidth(ImageCanvas.this) + x];

            // The pixel value is translated into a color using
            // the default color model.
            label.setText(x + "," + y + ":     " + cm.getRed(p) + " " 
                          + cm.getGreen(p) + " " + cm.getBlue(p) 
                          + "   a("+cm.getAlpha(p) + ")");
        }
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            int x = evt.getX(), y = evt.getY();
            if (pixels == null ||
                x >= image.getWidth(ImageCanvas.this) || 
                y >= image.getHeight(ImageCanvas.this)) {
                return;
            }
            targetPixelValue = pixels[y * image.getWidth(ImageCanvas.this) + x];
            paintTargetPixels = true;
            repaint();
        }

        public void mouseReleased(MouseEvent evt) {
            paintTargetPixels = false;
            repaint();
        }
    }
}
