import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener {
    String[] colorNames = {"red", "green", "blue"};
    Color[] colorValues = {Color.red, Color.green, Color.blue};
    MainButton[] buttons = new MainButton[colorNames.length];
    Canvas cv = new Canvas();

    Main() {
        super("Canvas Example");
        Panel p = new Panel(new GridLayout(1, 0));
        for (int i=0; i<colorNames.length; i++) {
            p.add(buttons[i] = new MainButton(colorNames[i], colorValues[i]));
            buttons[i].addActionListener(this);
        }
        add(p, BorderLayout.SOUTH);
        cv.setSize(150, 150);
        add(cv, BorderLayout.CENTER);
        pack();
        show();
    }

    // When a MainButton is clicked, update color of main canvas
    public void actionPerformed(ActionEvent evt) {
        for (int i=0; i<colorNames.length; i++) {
            if (colorNames[i].equals(evt.getActionCommand())) {
                buttons[i].setEnabled(false);
                cv.setBackground(colorValues[i]);
                cv.repaint();
            } else {
                buttons[i].setEnabled(true);
            }
        }
    }

    static public void main(String[] args) {
        new Main();
    }
}

class MainButton extends Canvas implements Runnable {
    boolean on;
    boolean engaged;
    String label;
    Thread timerThread;
    ActionListener actionListener;

    MainButton(String label, Color color) {
        this.label = label;
        (timerThread = new Thread(this)).start();
        setSize(40, 40);
        setForeground(color);
        setBackground(Color.gray);

        addMouseListener(new MouseListener());
    }

    public void setEnabled(boolean on) {
        super.setEnabled(on);
        if (on) {
            if (timerThread == null) {
                (timerThread = new Thread(this)).start();
            }
        } else {
            timerThread = null;
        }
        repaint();
    }

    public void addActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }

    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = getSize().width;
        int h = getSize().height;
        int ovalSize = Math.min(w/3, h/3);

        g.clearRect(0, 0, w, h);
        if (isEnabled()) {
            if (engaged || on) {
                g.fillOval((w-ovalSize)/2, (h-ovalSize)/2, 
                    ovalSize, ovalSize);
            } else {
                g.drawOval((w-ovalSize)/2, (h-ovalSize)/2, 
                    ovalSize, ovalSize);
            }
        } else {
            g.setColor(Color.white);
            g.drawOval((w-ovalSize)/2, (h-ovalSize)/2, ovalSize, ovalSize);
        }
        g.setColor(getBackground());
        g.draw3DRect(0, 0, w-1, h-1, !engaged);
        g.draw3DRect(1, 1, w-2, h-2, !engaged);
    }


    // Mouse event listener
    class MouseListener extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            engaged = true;
            repaint();
        }

        public void mouseReleased(MouseEvent evt) {
            engaged = false;
            repaint();

            if (actionListener != null) {
	        // Create action event for listener to process
                ActionEvent action = 
                    new ActionEvent(this, ActionEvent.ACTION_PERFORMED, label);
                actionListener.actionPerformed(action);
            }
        }
    }

    public void run() {
        while (timerThread == Thread.currentThread()) {
            on = !on;
            repaint();
            try { Thread.sleep(1000); } catch (Exception e) {};
        }
    }
}
