import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;

class Main extends Frame implements ActionListener {
    Button b = new Button("See Current Time");

    Main() {
        super("Dialog Example");

        // Listener for events.
        b.addActionListener(this);
    
        // Layout components.
        add(b, BorderLayout.NORTH);
        pack();
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        // Get the current time in the default locale.
        String time = DateFormat.getTimeInstance(
            DateFormat.LONG).format(new Date());

        // Create the message dialog.
        new MessageDialog(this, "The time is now - "+time);
    }

    public static void main(String[] args) {
        new Main();
    }
}

class MessageDialog extends Dialog implements ActionListener {
    MessageDialog(Frame f, String msg) {
        super(f, "Message", true);

        // Create the components.
        Button b = new Button("OK");
        Label l = new Label(msg, Label.CENTER) {
            // This adds some space around the text.
            public Dimension getPreferredSize() {
                Dimension d = super.getPreferredSize();
                return new Dimension(d.width+40, d.height+40);
            }
        };

        // Listen for events.
        b.addActionListener(this);
        addWindowListener(new WindowEventHandler());

        // Layout components.
        add(l, BorderLayout.NORTH);
        add(b, BorderLayout.SOUTH);

        pack();
        Dimension myDim = getSize();
        Dimension frameDim = f.getSize();
        Dimension screenSize = getToolkit().getScreenSize();
        Point loc = f.getLocation();
        
        // Center the dialog w.r.t. the frame.
        loc.translate((frameDim.width-myDim.width)/2, 
            (frameDim.height-myDim.height)/2);

        // Ensure that slave is withing screen bounds.
        loc.x = Math.max(0, Math.min(loc.x, screenSize.width-getSize().width));
        loc.y = Math.max(0, 
            Math.min(loc.y, screenSize.width-getSize().height));

        setLocation(loc.x, loc.y);
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        dispose();
    }

    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent evt) {
            dispose();
        }
    }
}
