import java.awt.*;
import java.awt.event.*;

class Main11 extends Frame {
    Main11() {
        super("Event example using Java 1.1");

        // Create listeners for all components
        MouseListener ml = new MouseEventHandler();
        MouseMotionListener mml = new MouseMotionEventHandler();
        FocusListener fl = new FocusEventHandler();
        ComponentListener cpl = new ComponentEventHandler();
        KeyListener kl = new KeyEventHandler();

        ContainerListener ctl = new ContainerEventHandler();
        WindowListener wl = new WindowEventHandler();

        // Handles Action, Text, Item and Adjustement events
        SemanticEventHandler sl = new SemanticEventHandler();

        // Add listeners for frame, this will catch
        // events as components are added
        addMouseListener(ml);
        addMouseMotionListener(mml);
        addFocusListener(fl);
        addComponentListener(cpl);
        addKeyListener(kl);
        addContainerListener(ctl);
        addWindowListener(wl);

        // Menu bar with a menu and menu item.
        MenuBar mb = new MenuBar();
        Menu m = new Menu("Menu");
        CheckboxMenuItem cbm = new CheckboxMenuItem("CheckboxMenuItem");        
        m.add("MenuItem");
        m.add(cbm);
        mb.add(m);
        setMenuBar(mb);

        setLayout(new GridLayout(0, 3));

        // checkbox, grouped checkbox, label, text field, scrollbar, button
        
        Checkbox cb1, cb2;
        Label label;
        TextField textField;
        Button button;
        MyCanvas canvas;

        add(cb1 = new Checkbox("Checkbox"));
        add(cb2 = new Checkbox("Checkbox", new CheckboxGroup(), true));
        add(label = new Label("label"));
        add(textField = new TextField("TextField"));
        add(button = new Button("Button"));
        add(canvas = new MyCanvas());

        // scrollbar 
        Scrollbar sb = new Scrollbar(Scrollbar.HORIZONTAL);
        sb.setValues(50, 50, 0, 100);
        add(sb);

        // choice
        Choice choice = new Choice();
        choice.addItem("Choice");
        choice.addItem("a choice item");
        add(choice);

        // list
        List list = new List();
        list.addItem("List");
        list.addItem("a list item");
        add(list);
        
        // text area
        TextArea textArea = new TextArea("TextArea");
        textArea.setSize(100, 50);
        add(textArea);


        // Add listeners to components

        // Checkboxes
        cb1.addMouseListener(ml);
        cb1.addMouseMotionListener(mml);
        cb1.addFocusListener(fl);
        cb1.addComponentListener(cpl);
        cb1.addKeyListener(kl);
        cb1.addItemListener(sl);
        
        cb2.addMouseListener(ml);
        cb2.addMouseMotionListener(mml);
        cb2.addFocusListener(fl);
        cb2.addComponentListener(cpl);
        cb2.addKeyListener(kl);
        cb2.addItemListener(sl);

        // Choice
        choice.addMouseListener(ml);
        choice.addMouseMotionListener(mml);
        choice.addFocusListener(fl);
        choice.addComponentListener(cpl);
        choice.addKeyListener(kl);
        choice.addItemListener(sl);

        // Label
        label.addMouseListener(ml);
        label.addMouseMotionListener(mml);
        label.addFocusListener(fl);
        label.addComponentListener(cpl);
        label.addKeyListener(kl);

        // Button
        button.addMouseListener(ml);
        button.addMouseMotionListener(mml);
        button.addFocusListener(fl);
        button.addComponentListener(cpl);
        button.addKeyListener(kl);
        button.addActionListener(sl);

        // Scrollbar
        sb.addMouseListener(ml);
        sb.addMouseMotionListener(mml);
        sb.addFocusListener(fl);
        sb.addComponentListener(cpl);
        sb.addKeyListener(kl);
        sb.addAdjustmentListener(sl);
        
        // List
        list.addMouseListener(ml);
        list.addMouseMotionListener(mml);
        list.addFocusListener(fl);
        list.addComponentListener(cpl);
        list.addKeyListener(kl);
        list.addItemListener(sl);
        list.addActionListener(sl);

        // Text Field
        textField.addMouseListener(ml);
        textField.addMouseMotionListener(mml);
        textField.addFocusListener(fl);
        textField.addComponentListener(cpl);
        textField.addKeyListener(kl);
        textField.addActionListener(sl);
        textField.addTextListener(sl);

        // Text Area
        textArea.addMouseListener(ml);
        textArea.addMouseMotionListener(mml);
        textArea.addFocusListener(fl);
        textArea.addComponentListener(cpl);
        textArea.addKeyListener(kl);
        textArea.addTextListener(sl);

        // canvas
        canvas.addMouseListener(ml);
        canvas.addMouseMotionListener(mml);
        canvas.addFocusListener(fl);
        canvas.addComponentListener(cpl);
        canvas.addKeyListener(kl);

        // Checkbox Menu 
        cbm.addItemListener(sl);
        cbm.addActionListener(sl);

        // Menu
        m.addActionListener(sl);

        pack();
        show();
    }

    class MouseEventHandler implements MouseListener {
        void printEvent(MouseEvent evt) {
            System.out.print(" ("+evt.getX()+" "+evt.getY()+") w(");
            System.out.println(evt.getWhen()+")");
            System.out.print("m("+evt.getModifiers()+")"+" c(");
            System.out.println(evt.getClickCount()+")");
        }

        public void mouseClicked(MouseEvent evt) {
            System.out.print(" ("+evt.getX()+" "+evt.getY()+") w(");
            System.out.print("MOUSE_CLICKED"); 
            printEvent(evt);
        }

        public void mousePressed(MouseEvent evt) {
            System.out.print(" ("+evt.getX()+" "+evt.getY()+") w(");
            System.out.print("MOUSE_DOWN"); 
            printEvent(evt);
        }

        public void mouseReleased(MouseEvent evt) {
            System.out.print(" ("+evt.getX()+" "+evt.getY()+") w(");
            System.out.print("MOUSE_UP");
            printEvent(evt);
        }

        public void mouseEntered(MouseEvent evt) {
            System.out.print(" ("+evt.getX()+" "+evt.getY()+") w(");
            System.out.print("MOUSE_ENTER");
            printEvent(evt);
        }
        public void mouseExited(MouseEvent evt) {
            System.out.print(" ("+evt.getX()+" "+evt.getY()+") w(");
            System.out.print("MOUSE_EXIT");
            printEvent(evt);
        }
    }

    class MouseMotionEventHandler implements MouseMotionListener {
        void printEvent(MouseEvent evt) {
            System.out.print(" ("+evt.getX()+" "+evt.getY()+") w(");
            System.out.println(evt.getWhen()+")");
            System.out.print("m("+evt.getModifiers()+")"+" c(");
            System.out.println(evt.getClickCount()+")");
        }
        public void mouseDragged(MouseEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("MOUSE_DRAG"); 
            printEvent(evt);
        }
        public void mouseMoved(MouseEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("MOUSE_MOVE");
            printEvent(evt);
        }
    }

    // Implements methods for Action, Text, Item, Adjustment
    class SemanticEventHandler implements ActionListener, ItemListener,
        TextListener, AdjustmentListener {
        public void actionPerformed(ActionEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("ACTION_EVENT");
            System.out.println(" cmd(" + evt.getActionCommand() + ")");
        }

        public void itemStateChanged(ItemEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            switch (evt.getStateChange()) {
            case ItemEvent.SELECTED:
                System.out.print("ITEM_SELECT");
                break;

            case ItemEvent.DESELECTED:
                System.out.print("ITEM_DESELECT");
            }
            System.out.println(" item( " + evt.getItem() + ")");
        }

        public void adjustmentValueChanged(AdjustmentEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            switch (evt.getID()) {
            case AdjustmentEvent.TRACK:
                System.out.print("SCROLL_ABSOLUTE"); 
                break;
            case AdjustmentEvent.UNIT_INCREMENT:
                System.out.print("SCROLL_LINE_DOWN");
                break;
            case AdjustmentEvent.UNIT_DECREMENT:
                System.out.print("SCROLL_LINE_UP"); 
                break;
            case AdjustmentEvent.BLOCK_DECREMENT:
                System.out.print("SCROLL_PAGE_DOWN"); 
                break;
            case AdjustmentEvent.BLOCK_INCREMENT:
                System.out.print("SCROLL_PAGE_UP"); 
                break;
            }
            System.out.print(" value (" + evt.getValue() + ")");
            System.out.println(" adjustable (" + evt.getAdjustable() + ")");
        }

        public void textValueChanged(TextEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.println("TEXT_VALUE_CHANGED");
        }
    }

    class WindowEventHandler implements WindowListener {
        public void windowDeiconified(WindowEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("WINDOW_DEICONIFY"); 
            System.out.println(" window (" + evt.getWindow() + ")");
        }
        public void windowClosing(WindowEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("WINDOW_DESTROY"); 
            System.out.println(" window (" + evt.getWindow() + ")");
        }
        public void windowClosed(WindowEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("WINDOW_CLOSED");
            System.out.println(" window (" + evt.getWindow() + ")");
        }
        public void windowActivated(WindowEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("WINDOW_EXPOSE"); 
            System.out.println(" window (" + evt.getWindow() + ")");
        }
        public void windowDeactivated(WindowEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("WINDOW_DEACTIVATED");
            System.out.println(" window (" + evt.getWindow() + ")");
        }
        public void windowIconified(WindowEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("WINDOW_ICONIFY"); 
            System.out.println(" window (" + evt.getWindow() + ")");
        }
        public void windowOpened(WindowEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("WINDOW_OPENED"); 
            System.out.println(" window (" + evt.getWindow() + ")");
        }
    }

    class ComponentEventHandler implements ComponentListener {
        public void componentMoved(ComponentEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("COMPONENT_MOVED"); 
            System.out.println(" component(" + evt.getComponent() + ")");
        }
        public void componentResized(ComponentEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("COMPONENT_RESIZED"); 
            System.out.println(" component(" + evt.getComponent() + ")");
        } 
        public void componentShown(ComponentEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("COMPONENT_SHOWN"); 
            System.out.println(" component(" + evt.getComponent() + ")");
        }
        public void componentHidden(ComponentEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("COMPONENT_HIDDEN"); 
            System.out.println(" component(" + evt.getComponent() + ")");
        }
    }

    class ContainerEventHandler implements ContainerListener {
        public void componentAdded(ContainerEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("CONTAINER_ADDED");
            System.out.println(" container("+evt.getContainer()+") child(" +
                               evt.getChild() +")");
        }
        public void componentRemoved(ContainerEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("CONTAINER_REMOVED");
            System.out.println(" container("+evt.getContainer()+") child(" +
                               evt.getChild() +")");
        }
    }

    class FocusEventHandler implements FocusListener {
        public void focusGained(FocusEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("FOCUS_GAINED");
            System.out.println(evt.isTemporary() ? " temporary." : ".");
        }
        public void focusLost(FocusEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("FOCUS_LOST");
            System.out.println(evt.isTemporary() ? " temporary." : ".");
        }
    }

    class KeyEventHandler implements KeyListener {
        public void keyPressed(KeyEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            if (evt.isActionKey()) {
                System.out.print("KEY_ACTION"); 
            } else {
                System.out.print("KEY_PRESS"); 
            }
            System.out.print(" k("+(char)evt.getKeyCode()+") m(");
            System.out.println(
                KeyEvent.getKeyModifiersText(evt.getModifiers())+")");
        }

        public void keyReleased(KeyEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            if (evt.isActionKey()) {
                System.out.print("KEY_ACTION_RELEASE");
            } else {
                System.out.print("KEY_RELEASE"); 
            }
            System.out.print(" k("+(char)evt.getKeyCode()+") m(");
            System.out.println(
                KeyEvent.getKeyModifiersText(evt.getModifiers())+")");
        }

        public void keyTyped(KeyEvent evt) {
            System.out.print(evt.getSource().getClass() + " ");
            System.out.print("KEY_TYPED");
            System.out.print(" k("+(char)evt.getKeyCode()+") m(");
            System.out.println(
                KeyEvent.getKeyModifiersText(evt.getModifiers())+")");
        }
    }

    static public void main(String[] args) {
        new Main11();
    }
}

class MyCanvas extends Canvas {
    public void paint(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        g.drawString("Canvas", 
            (getSize().width-fm.stringWidth("Canvas"))/2, 
            (getSize().height-fm.getAscent())/2);
    }
}
