import java.awt.*;
import java.util.*;

class Main extends Frame {
    Main(boolean vertical, String title) {
        super(title);
        setLayout(new FlexLayout(vertical, 2));
        add(new Button("one"));
        add("h", new Button("two"));
        add("*", new Button("three"));
        add(new Button("four"));

        setSize(100, 200);
        show();
    }
    public static void main(String args[]) {
        new Main(true, "LayoutManager Example 1");
        new Main(false, "LayoutManager Example 2");
    }
}

class FlexLayout implements LayoutManager {
    int gap;
    boolean ver;
    Hashtable comps = new Hashtable();

    public FlexLayout(boolean v, int gap) {
        ver = v;
        this.gap = gap;
    }

    public void addLayoutComponent(String name, Component comp) {
        comps.put(comp, name);
    }

    public void removeLayoutComponent(Component comp) {
        comps.remove(comp);
    }

    public Dimension layoutSize(Container parent, boolean minimum) {
        Insets insets = parent.getInsets();
        int w = 0, h = 0, n;

        // Grab tree lock to make sure components are not removed while
        // getting their sizes
        synchronized (parent.getTreeLock()) {
            n = parent.getComponentCount();

            for (int i=0; i<n; i++) {
                Component c = parent.getComponent(i);
                Dimension d = minimum ? c.getMinimumSize() : 
                    c.getPreferredSize();

                if (ver) {
                    w = Math.max(w, d.width);
                    h += d.height;
                } else {
                    w += d.width;
                    h = Math.max(h, d.height);
                }
            }
        }
        if (n > 0) {
            if (ver) {
                h += (n-1)*gap;
            } else {
                w += (n-1)*gap;
            }        
        }
        return new Dimension(insets.left + insets.right + w,
                             insets.top + insets.bottom + h);
    }
    
    public Dimension minimumLayoutSize(Container parent) {
        return layoutSize(parent, true);
    }

    public Dimension preferredLayoutSize(Container parent) {
        return layoutSize(parent, false);
    }

    public boolean isFlexible(Component c, boolean v) {
        String name = (String)comps.get(c);
        if (name == null) {
            return false;
        } else if (name.equals("*")) {
            return true;
        } else if (v) {
            return name.equals("v");
        } 
        return name.equals("h");
    }

    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        Dimension dim = layoutSize(parent, false);
        
        // Grab tree lock to make sure components are not removed while
        // laying them out.
        synchronized (parent.getTreeLock()) {
            
            int n = parent.getComponentCount();
            int flexCnt = 0;
            int extra = 0;
            int add = 0;
            
            for (int i=0; i<n; i++) {
                if (isFlexible(parent.getComponent(i), ver)) {
                    flexCnt++;
                }
            }
            if (flexCnt > 0) {
                if (ver) {
                    extra = parent.getSize().height-dim.height;
                } else {
                    extra = parent.getSize().width-dim.width;
                }
            }
            add = extra/flexCnt;
            
            if (ver) {
                int y = insets.top;
                for (int i=0; i<n; i++) {
                    int x = insets.left;
                    Component c = parent.getComponent(i);
                    String name = (String)comps.get(c);
                    Dimension d = c.getPreferredSize();
                    
                    if (isFlexible(c, ver)) {
                        d.height += add;
                        extra -= add;
                        if (extra < 0) {    // adjust for round off error
                            d.height += extra;
                        }
                    } 
                    if (isFlexible(c, !ver)) {
                        d.width = parent.getSize().width - 
                            insets.left - insets.right;
                    } else {
                        x = (parent.getSize().width - d.width)/2;
                    }
                    c.setBounds(x, y, d.width, d.height);
                    y += d.height + gap;
                }
            } else {
                int x = insets.left;
                for (int i=0; i<n; i++) {
                    int y = insets.top;
                    Component c = parent.getComponent(i);
                    String name = (String)comps.get(c);
                    Dimension d = c.getPreferredSize();
                    
                    if (isFlexible(c, ver)) {
                        d.width += add;
                        extra -= add;
                        if (extra < 0) {    // adjust for round off error
                            d.width += extra;
                        }
                    } 
                    if (isFlexible(c, !ver)) {
                        d.height = parent.getSize().height - 
                            insets.top - insets.bottom;
                    } else {
                        y = (parent.getSize().height - d.height)/2;
                    }
                    c.setBounds(x, y, d.width, d.height);
                    x += d.width + gap;
                }
            }
        }
    }
    
    public String toString() {
        return getClass().getName() + "[ver=" + ver + ",gap=" + gap + "]";
    }
}
