import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class Main implements ActionListener {
    Main () {
        MenuFrame mf = new MenuFrame();

        // Now create a few menus.
        Menu menu1 = new Menu("File");
        menu1.add("New");
        menu1.add("Open...");
        mf.add(menu1);

        Menu menu2 = new Menu("Edit");
        menu2.add("Cut");
        menu2.add("Paste");
        mf.add(menu2);

        // Add self as listener
        mf.addActionListener(this);

        // Display menu
        mf.setVisible(true);
    }

    // Action Listener for Menu Frame
    public void actionPerformed(ActionEvent evt) {
        System.out.println("action performed " + evt);
    }

    static public void main(String[] args) {
        new Main();
    }
}

class MenuFrame extends Frame implements MenuContainer {
    Font font = new Font("Monospaced", Font.PLAIN, 14);
    FontMetrics fontM;
    transient ActionListener actionListener;

    // Contains all the menu in this container.
    Vector menus = new Vector();

    // Contains all the menu items in this container.
    Vector menuItems = null;

    MenuFrame() {
        super("MenuContainer Example");
        this.menus = menus;
        fontM = getFontMetrics(font);

        setResizable(false);

        // Add listener for handling mouse clicks
        addMouseListener(new MouseEventHandler());

        // Enable mask for action events
        enableEvents(ActionEvent.ACTION_EVENT_MASK);
    }

    // Override show() to get MenuFrame size first
    public void setVisible(boolean show) {
        if (!show) {
                super.setVisible(false);
        }
        // Create the peers so that getInsets() returns correct value
        addNotify();

        Dimension dim = getPreferredSize();
        Insets insets = getInsets();
        setSize(insets.left+dim.width, insets.top+dim.height);
        super.show();  // calls Frame.show()
    }

    // Override to calculate size of frame based on menu items
    public Dimension getPreferredSize() {
        Vector labels = getLabels();
        int width = 0;

        for (int i=0; i<labels.size(); i++) {
            width = Math.max(width, 
                             fontM.stringWidth((String)labels.elementAt(i)));
        }

        return new Dimension(width, labels.size() * fontM.getHeight());
    }
    
    public Font getFont() {
        return font;
    }

    // Generates a vector of labels.  The menu item labels are indented.
    Vector getLabels() {
        Vector result = new Vector();
        menuItems = new Vector();
        for (int i=0; i<menus.size(); i++) {
            Menu menu = (Menu)menus.elementAt(i);
            result.addElement(menu.getLabel());
            menuItems.addElement(null);

            for (int j=0; j<menu.getItemCount(); j++) {
                result.addElement("  "+menu.getItem(j).getLabel());
                menuItems.addElement(menu.getItem(j));
            }
        }
        return result;
    }
        
    // Paints the labels.
    public void paint(Graphics g) {
        Vector labels = getLabels();
        int y = fontM.getAscent();

        g.setFont(font);
        Insets insets = getInsets();
        for (int i=0; i<labels.size(); i++) {
            String label = (String)labels.elementAt(i);
    
            // Paint menu label blue
            if (label.charAt(0) != ' ') {
                g.setColor(Color.blue);
            }
            g.drawString(label, insets.left, insets.top+y);
            y += fontM.getHeight();
            g.setColor(Color.black);
        }
    }

    public void add(MenuComponent mcomp) {
        menus.addElement(mcomp);
        menuItems = null;        // invalidate old list
    }

    public void remove(MenuComponent mcomp) {
        menus.removeElement(mcomp);
        menuItems = null;        // invalidate old list
    }

    // ActionEvent methods
    public void addActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }

    protected void processEvent(AWTEvent evt) {
        if (evt instanceof ActionEvent) {
            processActionEvent((ActionEvent) evt);
        } else {
            super.processEvent(evt);
        }
    }
        
    protected void processActionEvent(ActionEvent evt) {
        if (actionListener != null) {
            actionListener.actionPerformed(evt);
        }
    }

    // Mouse Event Listener
    class MouseEventHandler extends MouseAdapter {
        // Determines which item was clicked on and fire an event for that item.
        public void mousePressed(MouseEvent evt) {
            Vector labels = getLabels();
            Insets insets = getInsets();
            int item = (evt.getY()-insets.top) / fontM.getHeight();
        
            // Don't generate events when the click it not on a label.
            if (item < labels.size()) {
                String label = ((String)labels.elementAt(item));

                //Don't generate event for menu labels.
                if (label.charAt(0) == ' ') {
                    ActionEvent action = 
                        new ActionEvent(menuItems.elementAt(item), 
                                        ActionEvent.ACTION_PERFORMED, 
                                        label.trim());
                    System.out.println("dispatched: " + action);
//                    dispatchEvent(action);
                    processEvent(action);
                }
            }
        }
    }
}
