import java.awt.*;
import java.util.*;

class Main {
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: java Main <image file>");
            System.exit(1);
        }

        // Get the image.
        Image image = new ImageReader(args[0]).image;

        // Get the print job object.
        PrintJob pj = 
            Toolkit.getDefaultToolkit().getPrintJob(null, "Title", null);

        if (pj == null) {
            System.err.println("The print request was cancelled.");
            System.exit(1);
        }

        // Get the pixel dimensions of the page.
        Dimension d = pj.getPageDimension();

        // The above is currently not working in Java 1.1.2.
        // The following is a workaround. Given the screen resolution, 
        // determine pixel size of 8.5x11 inch sheet of paper.
        int res = Toolkit.getDefaultToolkit().getScreenResolution();
        d = new Dimension((int)(res * 8.5), (int)(res * 11));

        // Reduce the dimension of the page due to margins around
        // the page.  
        d.width -= (int)(res * .75/*in*/);
        d.height -= (int)(res * .75/*in*/);

        // Get graphics context for first page.
        Graphics g = pj.getGraphics();
        
        // Paint the image.
        g.drawImage(image, (d.width-image.getWidth(null))/2, 
            d.height/2-image.getHeight(null), null);

        // Set up the string and font.
        String msg = "\u5496\u5561?";
        g.setFont(new Font("Serif", Font.PLAIN, 48));
        FontMetrics fm = g.getFontMetrics();

        // Draw the string in the center.
        g.drawString(msg, (d.width-fm.stringWidth(msg))/2, 
            d.height/2+fm.getAscent());
    
        g.dispose();

        // Start printing.
        pj.end();

        System.exit(0);
    }
}

class ImageReader extends Component {
    Image image;
    ImageReader(String filename) {
        image = getToolkit().getImage(filename);
        MediaTracker tracker = new MediaTracker(this);
        try {
            tracker.addImage(image, 0);
            tracker.waitForAll(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
