import java.awt.*;
import java.awt.event.*;

class FakeButton extends Component {
    boolean up;

    FakeButton(boolean up) {
        this.up = up;

        // Listen for mouse clicks.
        addMouseListener(new MouseEventHandler());
    }

    public void paint(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = getSize().width;
        int h = getSize().height;
        String label = up ? "Up Button" : "Down Button";

        // Paint control background.
        g.setColor(Main.colors[SystemColor.CONTROL]);
        g.fillRect(0, 0, w, h);
        
        // Paint 3D outline.
        draw3DOutline(g, up, w, h);

        // Adjust the position of the text.
        // 
        int adjust = up ? -1 : 0;

        // Paint component text.
        g.setColor(Main.colors[SystemColor.CONTROL_TEXT]);
        g.drawString(label, adjust+(w-fm.stringWidth(label))/2, 
            adjust+(h-fm.getHeight())/2 + fm.getAscent());
        
    }

    // Paint a 3D outline using the system colors.
    public void draw3DOutline(Graphics g, boolean up, int w, int h) {
        Color c1 = Main.colors[SystemColor.CONTROL_LT_HIGHLIGHT];
        Color c2 = Main.colors[SystemColor.CONTROL_HIGHLIGHT];
        Color c3 = Main.colors[SystemColor.CONTROL_SHADOW];
        Color c4 = Main.colors[SystemColor.CONTROL_DK_SHADOW];

        if (!up) {
            c1 = Main.colors[SystemColor.CONTROL_DK_SHADOW];
            c2 = Main.colors[SystemColor.CONTROL_SHADOW];
            c3 = Main.colors[SystemColor.CONTROL_LT_HIGHLIGHT];
            c4 = Main.colors[SystemColor.CONTROL_HIGHLIGHT];
        }

        // Paint outer left and top edge.
        g.setColor(c1);
        g.drawLine(0, 0, w-1, 0);
        g.drawLine(0, 0, 0, h-1);

        // Paint inner left and top edge.
        g.setColor(c2);
        g.drawLine(1, 1, w-2, 1);
        g.drawLine(1, 1, 1, h-2);

        // Paint inner shadow right and bottom edge.
        g.setColor(c3);
        g.drawLine(w-2, h-2, w-2, 1);
        g.drawLine(w-2, h-2, 1, h-2);

        // Paint outter shadow right and bottom edge.
        g.setColor(c4);
        g.drawLine(w-1, h-1, w-1, 0);
        g.drawLine(w-1, h-1, 0, h-1);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            up = !up;
            repaint();
        }
    }
}

