import java.awt.*;
import java.awt.event.*;

class FakeList extends Component {
    FontMetrics fm;
    int curItem;

    FakeList() {
        // Listen for mouse clicks.
        addMouseListener(new MouseEventHandler());
    }

    public void paint(Graphics g) {
        int w = getSize().width;
        int h = getSize().height;

        if (fm == null) {
            fm = g.getFontMetrics();
        }

        // Paint control background.
        g.setColor(Main.colors[SystemColor.CONTROL]);
        g.fillRect(0, 0, w, h);
        
        // Paint 3D outline.
        draw3DOutline(g, true, 0, 0, w, h);
        draw3DOutline(g, false, 2, 2, w-4, h-4);

        // Don't paint over the borders.
        g.setClip(4, 4, w-8, h-8);

        for (int i=0; i<=h/fm.getHeight(); i++) {
            String item = "item "+i;

            if (i == curItem) {
                // Paint the select item background.
                g.setColor(Main.colors[SystemColor.TEXT_HIGHLIGHT]);
                g.fillRect(0, 4+i*fm.getHeight(), w, fm.getHeight());

                g.setColor(Main.colors[SystemColor.TEXT_HIGHLIGHT_TEXT]);
            } else {
                // Paint the select item background.
                g.setColor(Main.colors[SystemColor.TEXT]);
                g.fillRect(0, 4+i*fm.getHeight(), w, fm.getHeight());

                g.setColor(Main.colors[SystemColor.TEXT_TEXT]);
            }

            // Paint item label.
            // The 4 is to adjust for the border.
            g.drawString(item, 4+(h-fm.getHeight())/2, 
                4+i*fm.getHeight() + fm.getAscent());
        }
    }

    // Paint a 3D outline using the system colors.
    public void draw3DOutline(Graphics g, boolean up, 
            int x, int y, int w, int h) {
        Color c1 = Main.colors[SystemColor.CONTROL_LT_HIGHLIGHT];
        Color c2 = Main.colors[SystemColor.CONTROL_HIGHLIGHT];
        Color c3 = Main.colors[SystemColor.CONTROL_SHADOW];
        Color c4 = Main.colors[SystemColor.CONTROL_DK_SHADOW];

        if (!up) {
            c1 = Main.colors[SystemColor.CONTROL_DK_SHADOW];
            c2 = Main.colors[SystemColor.CONTROL_SHADOW];
            c3 = Main.colors[SystemColor.CONTROL_LT_HIGHLIGHT];
            c4 = Main.colors[SystemColor.CONTROL_HIGHLIGHT];
        }

        // Paint outer left and top edge.
        g.setColor(c1);
        g.drawLine(x, y, x+w-1, y);
        g.drawLine(x, y, x, y+h-1);

        // Paint inner left and top edge.
        g.setColor(c2);
        g.drawLine(x+1, y+1, x+w-2, y+1);
        g.drawLine(x+1, y+1, x+1, y+h-2);

        // Paint inner shadow right and bottom edge.
        g.setColor(c3);
        g.drawLine(x+w-2, y+h-2, x+w-2, y+1);
        g.drawLine(x+w-2, y+h-2, x+1, y+h-2);

        // Paint outer shadow right and bottom edge.
        g.setColor(c4);
        g.drawLine(x+w-1, y+h-1, x+w-1, y);
        g.drawLine(x+w-1, y+h-1, x, y+h-1);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            if (fm != null) {
                // The 4 is to adjust for the border pixels.
                curItem = (evt.getY()-4)/fm.getHeight();
                repaint();
            }
        }
    }
}

