import java.awt.*;

class FakeWindow extends Container {
    int bgColor;
    int textColor;
    boolean active;
    
    FakeScrollbar scrollbar = new FakeScrollbar();
    FakeWindowHeader header;
    
    FakeWindow(int bgColor, int textColor, boolean active) {
        this.bgColor = bgColor;
        this.textColor = textColor;
        this.active = active;

        setLayout(new BorderLayout());
        add(scrollbar, BorderLayout.EAST);
        header = new FakeWindowHeader(active);
        add(header, BorderLayout.NORTH);
    }
    
    public Insets getInsets() {
        return new Insets(4, 4, 4, 4);
    }

    public void paint(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = getSize().width;
        int h = getSize().height;

        // Paint window background color.
        g.setColor(Main.colors[bgColor]);
        g.fillRect(0, 0, w, h);

        if (active) {
            // Paint active window border.
            g.setColor(Main.colors[SystemColor.ACTIVE_CAPTION_BORDER]);
        } else {
            // Paint inactive window border.
            g.setColor(Main.colors[SystemColor.INACTIVE_CAPTION_BORDER]);
        }
        g.drawRect(0, 0, w-1, h-1);
        g.drawRect(1, 1, w-2, h-2);

        // Paint window border.
        g.setColor(Main.colors[SystemColor.WINDOW_BORDER]);
        g.drawRect(2, 2, w-5, h-5);
        g.drawRect(3, 3, w-7, h-7);

        // Paint text color
        g.setColor(Main.colors[textColor]);
        g.drawString("Window Text", 40, 70);

        // Needed to paint lightweight container's children.
        super.paint(g);
    }
}

class FakeWindowHeader extends Container {
    FakeWindowHeader(boolean active) {
        setLayout(new GridLayout(0, 1));

        add(new FakeTitleBar(active));
        add(new FakeMenuBar());
    }
}
