import java.awt.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.net.*;

class Main {
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: java Main <name of bean class file>");
            System.exit(1);
        }

        try {
            // Load the bean and get the bean-info.
            // (See the Introspector class example for FileClassLoader 
            // source code.)
            Class beanClass = FileClassLoader.load(args[0]);
            BeanInfo beanInfo = Introspector.getBeanInfo(beanClass);

            // Instantiate the bean.
            Object bean = beanClass.newInstance(); 

            // Now print the indexed properties and their values.
            PropertyDescriptor pds[] = beanInfo.getPropertyDescriptors();
            for (int i=0; i<pds.length; i++) {
                if (pds[i] instanceof IndexedPropertyDescriptor) {
                    IndexedPropertyDescriptor ipd = 
                        (IndexedPropertyDescriptor)pds[i];
                    int arrayLength = 0;

                    // Get the accessor method objects.
                    Method reader = ipd.getReadMethod();
                    Method indexedReader = ipd.getIndexedReadMethod();
                    Method writer = ipd.getWriteMethod();
                    Method indexedWriter = ipd.getIndexedWriteMethod();

                    // If any of the accessor methods are missing,
                    // just skip the property.
                    if (reader == null || indexedReader == null
                        || writer == null || indexedWriter == null
                        || ipd.getIndexedPropertyType() != String.class) {
                        /* 
                           An alternative test is:
                             ipd.getPropertyType() != String[].class
                        */
                        continue;
                    }

                    // Print the name of the property
                    System.out.println(ipd.getName());

                  // 1) Read and print the array.
                    // Invoke the read accessor method and print the
                    // contents of the array.
                    Object[] readerArgs = {};
                    Object array = reader.invoke(bean, readerArgs);
                    arrayLength = Array.getLength(array);

                    System.out.print("  non-indexed reader: ");
                    for (int j=0; j<arrayLength; j++) {
                        Object element = Array.get(array, j);

                        System.out.print(element.toString() + " ");
                    }
                    System.out.println();
                    
                  // 2) Sort the array.
                    array = sort((String[])array);

                  // 3) Write the array.
                    // Invoke the write accessor method on the sorted array.
                    Object[] writerArgs = {array};
                    writer.invoke(bean, writerArgs);

                  // 4) Read and print the array elements.
                    // Invoke the indexed read accessor method and print the
                    // contents of the array.
                    System.out.print("      indexed reader: ");
                    for (int j=0; j<arrayLength; j++) {
                        Object[] indexedReaderArgs = {new Integer(j)};
                        Object element = 
                            indexedReader.invoke(bean, indexedReaderArgs);

                        System.out.print(element.toString() + " ");
                    }
                    System.out.println();

                  // 5) Write the array elements backwards
                    // Invoke the indexed write accessor method and print the
                    // contents of the array.
                    array = (String[])((String[])array).clone();
                    for (int j=0; j<arrayLength; j++) {
                        Object[] indexedWriterArgs = 
                            {new Integer(arrayLength-j-1), 
                                Array.get(array, j)};

                        indexedWriter.invoke(bean, indexedWriterArgs);
                    }
                    
                  // 6) Read and print the array again.
                    array = reader.invoke(bean, readerArgs);
                    arrayLength = Array.getLength(array);

                    System.out.print("  non-indexed reader: ");
                    for (int j=0; j<arrayLength; j++) {
                        Object element = Array.get(array, j);

                        System.out.print(element.toString() + " ");
                    }
                    System.out.println();
                }
            }
        } catch (IntrospectionException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        }
        System.exit(0);
    }

    // Prints s and pads the string with Math.max(1, width-s.length()) spaces.
    static void print(String s, int width) {
        System.out.print(s);
        for (int i=0; i<Math.max(1, (width-s.length())); i++) {
            System.out.print(" ");
        }
    }

    // Returns a sorted copy of the string array.
    // Uses a simple bubble sort algorithm.
    static String[] sort(String[] a) {
        a = (String[])a.clone();

        for (int i=a.length-1; i>=0; i--) {
            for (int j=0; j<i; j++) {
                if (a[j].compareTo(a[j+1]) > 0) {
                    String s = a[j];
                    a[j] = a[j+1];
                    a[j+1] = s;
                }
            }
        }
        return a;
    }
}
