import java.awt.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.net.*;

class Main {
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: java Main <name of bean class file>");
            System.exit(1);
        }

        try {
            // Load the bean class and get the property descriptors.
            // (See the Introspector class example for FileClassLoader 
            // source code.)
            Class beanClass = FileClassLoader.load(args[0]);
            BeanInfo beanInfo = Introspector.getBeanInfo(beanClass);
            PropertyDescriptor pds[] = beanInfo.getPropertyDescriptors();
            Frame frame = null;

            for (int i=0; i<pds.length; i++) {
                PropertyEditor peditor = null;
                Class peClass = pds[i].getPropertyEditorClass();

                if (peClass != null) {
                    try {
                        // Instatiate the custom editor
                        peditor = (PropertyEditor)peClass.newInstance();
                    } catch (Exception e) {
                    }
                }
                if (peditor != null && peditor.supportsCustomEditor()) {
                    try {
                        // Instantiate the bean.
                        Object bean = beanClass.newInstance();
                        Method getter = pds[i].getReadMethod();
                        Object arguments[] = { };
                        peditor.setValue(getter.invoke(bean, arguments));
        
                        // Create a frame for the custom editor.
                        if (frame == null) {
                            frame = new Frame("getPropertyEditorClass Example");
                            frame.setLayout(new GridBagLayout());
                        }
                        addComp(frame, pds[i].getName(), 
                            peditor.getCustomEditor());
                    } catch (InvocationTargetException e) {
                        e.printStackTrace();
                    } catch (InstantiationException e) {
                        e.printStackTrace();
                    } catch (IllegalAccessException e) {
                        e.printStackTrace();
                    }
                }
            }
            if (frame != null) {
                frame.pack();
                frame.show();
            }
        } catch (IntrospectionException e) {
            e.printStackTrace();
        }
    }

    static void addComp(Container cont, String label, Component comp) {
        GridBagLayout gbl = (GridBagLayout)cont.getLayout();
        GridBagConstraints c = new GridBagConstraints();

        // Add label.
        Label l= new Label(label);
        cont.add(l);
        gbl.setConstraints(comp, c);

        // Add component.
        cont.add(comp);
        c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        gbl.setConstraints(comp, c);
    }
}
