import java.awt.*;
import java.beans.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.net.*;

class Main {
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: java Main <name of bean class file>");
            System.exit(1);
        }

        try {
            // Load the bean and get the bean-info.
            // (See the Introspector class example for 
            // FileClassLoader source code.)
            Class beanClass = FileClassLoader.load(args[0]);
            BeanInfo beanInfo = Introspector.getBeanInfo(beanClass);

            // Instantiate the bean.
            Object bean = beanClass.newInstance(); 

            // Components are treated specially.
            if (Beans.isInstanceOf(bean, Component.class)) {
                Frame frame = new Frame();
                frame.add((Component)bean);
                frame.addNotify();
            }

            // Now print each property and its value.
            PropertyDescriptor pds[] = beanInfo.getPropertyDescriptors();
            for (int i=0; i<pds.length; i++) {
                Method reader = pds[i].getReadMethod();
                Object[] arguments = {};

                // Invoke the read accessor method.
                Object value = reader.invoke(bean, arguments);

                // Print the results.
                print(pds[i].getName(), 15);
                System.out.println(value);
            }
        } catch (IntrospectionException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        }

        System.exit(0);
    }

    // Prints s and pads the string with Math.max(1, width-s.length()) spaces.
    static void print(String s, int width) {
        System.out.print(s);
        for (int i=0; i<Math.max(1, (width-s.length())); i++) {
            System.out.print(" ");
        }
    }
}

